unit RAM;
interface

uses Classes, Tools;

Type TSpeicher=class
  private
    FData : Array of Byte;
    FProtect : Boolean;
    FProtectList : TList;
    FAddr : Integer;
    function GetUsed : Integer;
  protected
    function GetData(I : Integer) : Byte; virtual;
    procedure SetData(I : Integer; B : Byte); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; virtual;
    procedure LoadData(const St : TStream; StartAddr : Integer; Count : Integer = 0);
    procedure SaveData(const St : TStream; StartAddr : Integer; Count : Integer = 0);
    procedure SetSize(I : Integer);
    property Data[I : Integer] : Byte read GetData write SetData; default;
    property ProtectList : TList read FProtectList;
    property Protect : Boolean read FProtect write FProtect;
    property Addr : Integer read FAddr write FAddr;
    property Used : Integer read GetUsed;
end;

Type TSFRArea=class(TSpeicher)
  private
    DPL,DPH : Array[0..7] of Byte;
    FOnSBUFWrite : TNotifyEvent;
    RegSave : Array[0..5] of Byte;
    FSecond : Boolean;
 protected
    function GetData(I : Integer) : Byte; override;
    procedure SetData(I : Integer; B : Byte); override;
  public
    Function Read2(Addr : Integer) : Byte;
    Procedure WritePinOnly(Addr : Integer; B : Byte);
    constructor Create(Second : Boolean = False);
    procedure Clear; override;
  published
    property OnSBUFWrite : TNotifyEvent read FOnSBUFWrite write FOnSBUFWrite;
end;

implementation

uses SysUtils, Dialogs, PrgConst;

{ TSpeicher }

constructor TSpeicher.Create;
begin
  inherited Create;
  FProtect:=False;
  FProtectList:=TList.Create;
  Clear;
end;

destructor TSpeicher.Destroy;
begin
  FProtectList.Free;
  inherited Destroy;
end;

procedure TSpeicher.Clear;
begin
  SetLength(FData,0);
end;

function TSpeicher.GetData(I: Integer): Byte;
begin
  If I<0 then raise ESystemError.Create('Zugriffsversuch auf eine negative Adresse');
  If I>=length(FData) then result:=0 else result:=FData[I];
end;

procedure TSpeicher.SetData(I: Integer; B: Byte);
Var OldSize : Integer;
begin
  If I<0 then raise ESystemError.Create('Zugriffsversuch auf eine negative Adresse');
  If FProtect and (FProtectList.IndexOf(Pointer(I))>=0) then
    MessageDlg(
      'Speicherschutzwarnung:'+#13+
      'Es wird durch den Befehl an Adresse 0x'+IntToHex(FAddr,4)+' schreibend auf '+
      'die geschtzte Speicherstelle 0x'+IntToHex(I,2)+' zugegriffen.',mtWarning,[mbOK],0);
  If I>=length(FData) then begin
    OldSize:=length(FData);
    SetLength(FData,I+1);
    FillChar(FData[OldSize],(I+1)-OldSize,0);
  end;
  FData[I]:=B;
end;

procedure TSpeicher.LoadData(const St: TStream; StartAddr, Count: Integer);
Var X : Integer;
    B : Byte;
begin
  If Count=0 then Count:=St.Size-St.Position;
  For X:=0 to Count-1 do begin St.Read(B,1); SetData(StartAddr+X,B); end;
end;

procedure TSpeicher.SaveData(const St: TStream; StartAddr, Count: Integer);
Var X : Integer;
    B : Byte;
begin
  If Count=0 then Count:=length(FData)-StartAddr;
  For X:=0 To Count-1 do begin B:=GetData(StartAddr+X); St.Write(B,1); end;
end;

function TSpeicher.GetUsed: Integer;
begin
  result:=length(FData);
end;

procedure TSpeicher.SetSize(I: Integer);
Var OldSize : Integer;
begin
  If I<=length(FData) then exit;
  OldSize:=length(FData);
  SetLength(FData,I);
  FillChar(FData[OldSize],I-OldSize,0);
end;

{ TSFRArea }

constructor TSFRArea.Create(Second : Boolean);
Var X : Integer;
begin
  inherited Create; FSecond:=Second;
  For X:=0 to 7 do begin DPL[X]:=0; DPH[X]:=0; end;
end;

function TSFRArea.GetData(I: Integer): Byte;
begin
  Case I of
    RegDPL : result:=DPL[GetData(RegDPSEL) mod 8];
    RegDPH : result:=DPH[GetData(RegDPSEL) mod 8];
    else result:=inherited GetData(I);
  end;
end;

procedure TSFRArea.SetData(I: Integer; B: Byte);
Var A : Byte;
begin
  Case I of
    RegDPL  : DPL[GetData(RegDPSEL) mod 8]:=B;
    RegDPH  : DPH[GetData(RegDPSEL) mod 8]:=B;
    RegPCON : begin
                A:=GetData(RegPCON);
                If (A and 1 <> 1) and (B and 32 = 32) then B:=B xor 32;
                If (A and 2 <> 2) and (B and 64 = 64) then B:=B xor 64;
                inherited SetData(RegPCON,B);
              end;
    RegSBUF : begin
                inherited SetData(RegSBUF,B);
                If Assigned(FOnSBUFWrite) then FOnSBUFWrite(self);
              end;
    RegP0 : begin RegSave[0]:=B; inherited SetData(I,B); end;
    RegP1 : begin RegSave[1]:=B; inherited SetData(I,B); end;
    RegP2 : begin RegSave[2]:=B; inherited SetData(I,B); end;
    RegP3 : begin RegSave[3]:=B; inherited SetData(I,B); end;
    RegP4 : begin RegSave[4]:=B; inherited SetData(I,B); end;
    RegP5 : begin RegSave[5]:=B; inherited SetData(I,B); end;
    RegADDATL : begin
                  inherited SetData(RegADCON0,inherited GetData(RegADCON0) or 16);
                  inherited SetData(I,B);
                end;
    RegADCON0,
    RegADCON1: begin
                  inherited SetData(I,B);
                  inherited SetData(RegADCON0,(B mod 4) or (inherited GetData(RegADCON0) and $FC));
                  inherited SetData(RegADCON1,(B mod 4) or (inherited GetData(RegADCON1) and $FC));
                end;
    else inherited SetData(I,B);
  end;
end;

function TSFRArea.Read2(Addr: Integer): Byte;
begin
  Case Addr of
    RegP0 : result:=RegSave[0];
    RegP1 : result:=RegSave[1];
    RegP2 : result:=RegSave[2];
    RegP3 : result:=RegSave[3];
    RegP4 : result:=RegSave[4];
    RegP5 : result:=RegSave[5];
    else result:=GetData(Addr);
  end;
end;

procedure TSFRArea.Clear;
Var X : Byte;
begin
  inherited Clear;
  For X:=0 to 5 do RegSave[X]:=0;
end;

procedure TSFRArea.WritePinOnly(Addr: Integer; B: Byte);
begin
  inherited SetData(Addr,B);
end;

end.
